@@ -64,6 +64,7 @@ Core::Core(QObject* parent)
 
 Core::~Core()
 {
+    qCDebug(POWERDEVIL) << "Core unloading";
     // Unload all actions before exiting, and clear the cache
     ActionPool::instance()->unloadAllActiveActions();
     ActionPool::instance()->clearCache();
@@ -138,6 +139,21 @@ void Core::onBackendReady()
         KIdleTime::instance()->simulateUserActivity();
     });
 
+    // Bug 354250: Simulate user activity when session becomes inactive,
+    // this keeps us from sending the computer to sleep when switching to an idle session.
+    // (this is just being lazy as it will result in us clearing everything
+    connect(PowerDevil::PolicyAgent::instance(), &PowerDevil::PolicyAgent::sessionActiveChanged, this, [this](bool active) {
+        if (active) {
+            // force reload profile so all actions re-register their idle timeouts
+            loadProfile(true /*force*/);
+        } else {
+            // Bug 354250: Keep us from sending the computer to sleep when switching
+            // to an idle session by removing all idle timeouts
+            KIdleTime::instance()->removeAllIdleTimeouts();
+            m_registeredActionTimeouts.clear();
+        }
+    });
+
     // Initialize the action pool, which will also load the needed startup actions.
     PowerDevil::ActionPool::instance()->init(this);
 
@@ -703,14 +719,12 @@ void Core::onBatteryRemainingTimeChanged(qulonglong time)
 void Core::onKIdleTimeoutReached(int identifier, int msec)
 {
     // Find which action(s) requested this idle timeout
-    for (QHash< Action*, QList< int > >::const_iterator i = m_registeredActionTimeouts.constBegin();
-         i != m_registeredActionTimeouts.constEnd(); ++i) {
+    for (auto i = m_registeredActionTimeouts.constBegin(), end = m_registeredActionTimeouts.constEnd(); i != end; ++i) {
         if (i.value().contains(identifier)) {
             i.key()->onIdleTimeout(msec);
+
             // And it will need to be awaken
-            if (!m_pendingResumeFromIdleActions.contains(i.key())) {
-                m_pendingResumeFromIdleActions.append(i.key());
-            }
+            m_pendingResumeFromIdleActions.insert(i.key());
             break;
         }
     }
@@ -761,7 +775,7 @@ int Core::currentChargePercent() const
 void Core::onResumingFromIdle()
 {
     // Wake up the actions in which an idle action was triggered
-    QList< Action* >::iterator i = m_pendingResumeFromIdleActions.begin();
+    auto i = m_pendingResumeFromIdleActions.begin();
     while (i != m_pendingResumeFromIdleActions.end()) {
         (*i)->onWakeupFromIdle();
         i = m_pendingResumeFromIdleActions.erase(i);
